package com.kingfore.config;

import org.springframework.data.mongodb.MongoDbFactory;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.SimpleMongoDbFactory;
import org.springframework.data.mongodb.core.mapping.MongoMappingContext;

import com.mongodb.MongoClient;
import com.mongodb.MongoClientOptions;
import com.mongodb.ServerAddress;

public abstract class AbstractMongoConfig {
	 private String host, database, username, password;
	 private int port;
	public String getHost() {
		return host;
	}
	public void setHost(String host) {
		this.host = host;
	}
	public String getDatabase() {
		return database;
	}
	public void setDatabase(String database) {
		this.database = database;
	}
	public String getUsername() {
		return username;
	}
	public void setUsername(String username) {
		this.username = username;
	}
	public String getPassword() {
		return password;
	}
	public void setPassword(String password) {
		this.password = password;
	}
	public int getPort() {
		return port;
	}
	public void setPort(int port) {
		this.port = port;
	}
	 
	/**
	    * 创建MongoDbFactory，不同数据源继承该方法创建对应的MongoDbFactory。
	    * @return
	    * @throws Exception
	    */
	    public MongoDbFactory mongoDbFactory() throws Exception {
	        ServerAddress serverAddress = new ServerAddress(host, port);
	        MongoClientOptions options = MongoClientOptions.builder()
	                .connectionsPerHost(100)
	                .socketTimeout(30000)
	                .connectTimeout(30000)
	                .build();
	        return new SimpleMongoDbFactory(new MongoClient(serverAddress, options), database);
	    }
	    /**
	    * 抽象方法，用于返回MongoTemplate
	    * @param context
	    * @return
	    * @throws Exception
	    */
	    abstract public MongoTemplate getMongoTemplate(MongoMappingContext context) throws Exception;
}
