package com.kingfore.config;

import java.util.concurrent.ThreadPoolExecutor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;

@Configuration
@EnableAsync
public class Config {
    private static final Logger logger = LoggerFactory.getLogger(Config.class);
    
    
    private static final int corePoolSize = 20;       		// 核心线程数（默认线程数）
	private static final int maxPoolSize = 20;			    // 最大线程数
	private static final int keepAliveTime = 10;			// 允许线程空闲时间（单位：默认为秒）
	private static final int queueCapacity = 200;			// 缓冲队列数
	private static final String threadNamePrefix = "Async-Service-"; // 线程池名前缀
    
	@Bean("taskExecutor-iot") // bean的名称，默认为首字母小写的方法名
	public ThreadPoolTaskExecutor taskExecutor(){
		ThreadPoolTaskExecutor executor = new ThreadPoolTaskExecutor();
		executor.setCorePoolSize(corePoolSize);   
		executor.setMaxPoolSize(maxPoolSize);
		executor.setQueueCapacity(queueCapacity);
		executor.setKeepAliveSeconds(keepAliveTime);
		executor.setThreadNamePrefix(threadNamePrefix);
		
		// 线程池对拒绝任务的处理策略
		executor.setRejectedExecutionHandler(new ThreadPoolExecutor.CallerRunsPolicy());
		// 初始化
		executor.initialize();
		logger.info("线程池配置完成......");
		return executor;
	}
	
	//influx
	
//	@Value("${spring.influx.url}")
//	private  String url;
//	@Value("${spring.influx.user}")
//	private  String userName;
//	@Value("${spring.influx.password}")
//	private  String password;
//	
//	@Value("${spring.influx.data-base}")
//	private  String database;
//	@Value("${spring.influx.retention-policy}")
//	private  String retentionPolicy;
//	@Bean("InfluxDBKF") // bean的名称，默认为首字母小写的方法名
//	public InfluxDBKF InfluxDBKFInit(){
//	
//		InfluxDBKF conn=new InfluxDBKF(userName,password,url,database,retentionPolicy);
//		logger.info("influx配置完成......");
//		return conn;
//		
//	}
	
    
}
