package com.kingfore.listener;

import com.alibaba.fastjson2.JSONArray;
import com.kingfore.service.impl.DataServiceImpl;
import com.kingfore.service.impl.MqttReviceImpl;
import net.dreamlu.iot.mqtt.codec.MqttPublishMessage;
import net.dreamlu.iot.mqtt.core.client.IMqttClientMessageListener;
import net.dreamlu.iot.mqtt.spring.client.MqttClientSubscribe;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.tio.core.ChannelContext;

import java.nio.charset.StandardCharsets;
import java.util.Date;

/**
 * 客户端消息监听的另一种方式
 *
 * @author L.cm
 */
@Service
@MqttClientSubscribe("${topic1}")
public class MqttClientMessageListener implements IMqttClientMessageListener {
    private static final Logger logger = LoggerFactory.getLogger(MqttClientMessageListener.class);

    @Autowired
    private MqttReviceImpl mqttRevice;
    @Autowired
    private DataServiceImpl dataService;

    @Value("${mqtt.client.ip}")
    private String ip;
    @Value("${mqtt.client.port}")
    private String port;

    @Override
    public void onMessage(ChannelContext context, String topic, MqttPublishMessage message, byte[] payload) {
        logger.info("接收消息主题 : " + topic);
//        logger.info("接收消息内容 : " + new String(payload, StandardCharsets.UTF_8));
        //logger.info("接收消息时间 : " + new Date());

        // 后续业务逻辑可以在这里做分发 根据topic
        String content = new String(payload, StandardCharsets.UTF_8);
        JSONArray json = JSONArray.of(content);
//        logger.info("接收消息内容: "+json.size()+"个设备");
//        mqttRevice.updateRealData(ip,port,topic,new Date(),content);

        dataService.updataRealData(ip,port,topic,content,new Date());
    }
}

