package com.kingfore.service.impl;


import com.alibaba.fastjson2.JSONArray;
import com.kingfore.utils.BeanMapUtils;
import com.kingfore.utils.CommonUtils;
import com.kingfore.utils.DateUtils;
import com.mongodb.bulk.BulkWriteResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.mongodb.core.BulkOperations;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@EnableScheduling
public class MqttReviceImpl {
    private static final Logger logger = LoggerFactory.getLogger(MqttReviceImpl.class);
    @Autowired
    MongoTemplate mongoTemplate;

    @Autowired
    @Qualifier(value = "realDataMongoTemplate")
    private MongoTemplate realDataMongoTemplate;

    @Autowired
    @Qualifier(value = "historyDataMongoTemplate")
    private  MongoTemplate historymongoTemplate;

    private static MqttReviceImpl mqttReviceImpl;
    @PostConstruct
    public void init() {
        mqttReviceImpl = this;
        mqttReviceImpl.realDataMongoTemplate = this.realDataMongoTemplate;
        mqttReviceImpl.historymongoTemplate = this.historymongoTemplate;
    }


    public Object reflect(Object obj, Map dataMap) throws Exception {

        Map objMap = BeanMapUtils.beanToMap(obj);
        for (Object key : objMap.keySet()) {
            if (dataMap.containsKey(key)) {
                objMap.put(key, dataMap.get(key));
            }
        }
        return BeanMapUtils.mapToBean(objMap, obj.getClass());
    }
//
//    @Getter
//    @Setter
//    List<Measurement> measurementList = new ArrayList<>();
//
//    @Cacheable(cacheNames = "measurement")
//    public List<Measurement> getMeasurement() {
//        if (measurementList.isEmpty()) {
//            this.measurementList = measurementDao.list();
//        }
//        return this.measurementList;
//    }
//
//    @CachePut(cacheNames = "measurement")
//    @Scheduled(fixedDelay = 600000)
//    public void refreshMeasurement() {
//        this.measurementList = measurementDao.list();
//        logger.info("更新仪器仪表");
//    }
//
//    @Getter
//    @Setter
//    List<DatacodeExtVO> dataCodeExtList = new ArrayList<>();
//
//    @Cacheable(cacheNames = "dataCodeExt")
//    public List<DatacodeExtVO> getDataCodeExt() {
//        if (dataCodeExtList.isEmpty()) {
//            this.dataCodeExtList = datacodeExtDao.getBaseMapper().getAllByDataCodeExt();
//        }
//        return this.dataCodeExtList;
//    }
//
//    @CachePut(cacheNames = "dataCodeExt")
//    @Scheduled(fixedDelay = 600000)
//    public void refreshDataCodeExt() {
//        this.dataCodeExtList = datacodeExtDao.getBaseMapper().getAllByDataCodeExt();
//        logger.info("更新模型");
//    }

    public void updateRealData(String host, String port, String topic, Date date, String content) {
        //解析topic
        Map<String, Object> topicMap = CommonUtils.expTopic(topic);
        updateMongoDBRealData(host, port, topicMap, new DateUtils().getNewDate(date), content);

    }

    public void updateMongoDBRealData(String host, String port, Map<String, Object> topicMap, Date date, String content) {
        try {
            //解析topic
            String deviceName = (String) topicMap.get("deviceName");
            String readWrite = (String) topicMap.get("readWrite");
            String dataPointType = (String) topicMap.get("dataPointType");


            BulkOperations ops = mongoTemplate.bulkOps(BulkOperations.BulkMode.UNORDERED, "realData");
            Query qu = Query.query(Criteria.where("deviceName").is(deviceName));
            List<Map> deviceAll = mongoTemplate.find(qu, Map.class, "realData");

            //查询项目
//            Project project = StrUtil.isNotBlank(projectCode)?getProject().stream().filter(f->f.getProjectCode().equals(projectCode) ).findFirst().get():new Project();
            //查询模型
//            List<DatacodeExtVO> datacodeExtList = StrUtil.isNotBlank(dataPointType) ? getDataCodeExt().stream().collect(Collectors.toList()) : new ArrayList();

            int updateCount = 0;
            int saveCount = 0;
            //判断数据点类型(bg为数字孪生数据，单独处理)
            if (dataPointType.equals("wr")) {
                JSONArray json = JSONArray.of(content);
                for (Object m : json) {
                    String item = (String) m;
                    updateCount = setOpt(item, deviceAll, ops, host, port, date, deviceName, dataPointType, updateCount, saveCount);
                }
            } else {
                updateCount = setOpt(content, deviceAll, ops, host, port, date, deviceName, dataPointType, updateCount, saveCount);
            }

            if (updateCount != 0) {
                BulkWriteResult result = ops.execute();
                logger.info("更新实时记录：" + updateCount);
            }

        } catch (Exception e) {
            e.printStackTrace();
            String error = e.getMessage();
            logger.error(error);
            throw new RuntimeException(e.getMessage());
        }
    }

    public int setOpt(String content, List<Map> deviceAll, BulkOperations ops,
                      String host, String port, Date date,
                      String deviceName, String dataPointType,
                      int updateCount, int saveCount
    ) {
//        List<Map> items = JSON.parseArray(content, Map.class);
//        for (Map<String, Object> map : items) {
//            String device = (String) map.get("Device");
//            Map<String, Object> data = (Map<String, Object>) map.get("Data");
//            for (String key : data.keySet()) {
////                //查询模型
////                DatacodeExtVO datacodeExtVO = datacodeExtList.stream().filter(f -> f.getDataCode().equals(key)).findAny().orElse(null);
////                if (ObjectUtils.isEmpty(datacodeExtVO)) continue;
////
////                //查询仪器仪表
////                Measurement measurement = getMeasurement().stream().filter(f -> f.getMeasurementCode().equals(device) && f.getSn().equals(deviceName)).findFirst().orElse(null);
//                Map<String, Object> groupOpt = deviceAll.stream().filter(f -> key.equals(f.get("dataCode")) && f.get("measurementCode").equals(measurement.getMeasurementCode())).findAny().orElse(null);
////                String dataName = datacodeExtVO.getDataName();
////                Integer rwStatus = datacodeExtVO.getRwStatus();
//                Object value = data.get(key);
//                if (groupOpt != null) {//有修改
//                    Update up = new Update();
//                    //写主题只更新读写状态，读主题更新值
//                    up.set("rwStatus", rwStatus);
//                    up.set("value", value);
//                    up.set("dt", date);
//                    up.set("onLine", "在线");
//                    up.set("measurementId", ObjectUtil.isNotEmpty(measurement) ? measurement.getId() : null);
//                    up.set("measurementName", ObjectUtil.isNotEmpty(measurement) ? measurement.getMeasurementName() : null);
//                    up.set("measurementType", ObjectUtil.isNotEmpty(measurement) ? measurement.getMeasurementType() : null);
//                    up.set("measurementCode", device);
//                    Query query = Query.query(Criteria.where("deviceName").is(deviceName).and("_id").is(deviceName + "_" + device + "_" + key));
//                    ops.updateOne(query, up);
//                    updateCount++;
//                } else {//无添加
//                    Map<String, Object> realData = new HashMap<>();
//                    realData.put("_id", deviceName + "_" + device + "_" + key);
//                    realData.put("dataCode", key);
//                    realData.put("dataName", dataName);
//                    realData.put("value", value);
//                    realData.put("dt", date);
////                    realData.put("equipmentId", ObjectUtil.isNotEmpty(project)?project.getId():null);
////                    realData.put("equipmentName", ObjectUtil.isNotEmpty(project)?project.getProjectName():null);
////                    realData.put("equipmentCode", ObjectUtil.isNotEmpty(project)?project.getProjectCode():null);
//                    realData.put("measurementId", ObjectUtil.isNotEmpty(measurement) ? measurement.getId() : null);
//                    realData.put("measurementName", ObjectUtil.isNotEmpty(measurement) ? measurement.getMeasurementName() : null);
//                    realData.put("measurementType", ObjectUtil.isNotEmpty(measurement) ? measurement.getMeasurementType() : null);
//                    realData.put("measurementCode", device);
//                    realData.put("dataPointType", dataPointType);
//                    realData.put("rwStatus", rwStatus);
//                    realData.put("history", "是");
//                    realData.put("deviceName", deviceName);
//                    realData.put("dataForm", "mqttCloud-bg");
//                    realData.put("onLine", "在线");
//                    realData.put("custom1", host);
//                    realData.put("custom2", port);
//                    ops.insert(realData);
//                    saveCount++;
//                }
//            }
//        }
        return saveCount + updateCount;
    }

}